//------------------------------------------------------------------------------
//	$Id: gen-zahlenraetsel.js,v 1.2 2019/01/01 04:12:54 wolf Exp $
//------------------------------------------------------------------------------
//
//	$Log: gen-zahlenraetsel.js,v $
//	Revision 1.2  2019/01/01 04:12:54  wolf
//	* cleanup
//
//	Revision 1.1  2018/12/20 01:43:17  wolf
//	Initial revision
//
//	Aus gen-zahlenraetsel.pl übernommen
//
//------------------------------------------------------------------------------

/*jsl:option explicit*/

'use strict';

var ZahlenraetselGenerator = {

//------------------------------------------------------------------------------
//	Constants
//------------------------------------------------------------------------------

'MinNumber':    2,
'MaxNumber': 9999,
'MaxTries':    20,

//------------------------------------------------------------------------------
//	Prime numbers
//------------------------------------------------------------------------------

'Prims': [2,3,5,7,11,13,17,19,23,29,31,37,41,43,47,53,59,61,67,71,73,79,83,89,97],

//------------------------------------------------------------------------------
//	factorize a number
//------------------------------------------------------------------------------

'factorize': function(number) {

	var factors = [];

	for (var i in this.Prims) {

		var prim = this.Prims[i];
		if (number < prim*prim) break;

		while (! (number % prim)) {

			factors.push(prim);
			number /= prim;
		}
	}

	if (number>=2) factors.push(number);
	return factors;
},

//------------------------------------------------------------------------------
//	find random p and q where p*q = product
//------------------------------------------------------------------------------

'findTwoFactors': function (product, minValue) {

	minValue = Math.max(minValue||4, 4);

	var factors = this.factorize(product);

	//----------------------------------------------------------------------
	//	factor 1
	//----------------------------------------------------------------------

	var p = factors.pop();

	while (p<minValue && factors.length>=3) {

		p *= factors.shift();
	}

	//----------------------------------------------------------------------
	//	factor 2
	//----------------------------------------------------------------------

	var q = 1;

	while (q<minValue && factors.length) {

		q *= factors.splice(Math.random(factors.length), 1)[0];
	}

	//----------------------------------------------------------------------
	//	distribute available prime factors randomly
	//----------------------------------------------------------------------

	while (factors.length) {

		if (Math.random()<0.5) {
			p *= factors.pop();
		} else {
			q *= factors.pop();
		}
	}

	//----------------------------------------------------------------------
	//	return small factor first
	//----------------------------------------------------------------------

	return [p, q].sort();
},

//------------------------------------------------------------------------------
//	find p, q, r, s where p*q - r*s = 2
//------------------------------------------------------------------------------

'Pairs': [
	208,	304,	306,	320,	323,	336,	338,	340,	350,	375,
	378,	390,	403,	406,	414,	416,	418,	432,	435,	440,
	448,	460,	462,	481,	484,	493,	494,	504,	510,	518,
	520,	525,	527,	544,	550,	558,	559,	570,	572,	574,
	576,	578,	592,	598,	600,	609,	625,	627,	638,	644,
	646,	648,	656,	658,	663,	665,	680,	682,	684,	686,
	688,	700,	702,	703,	713,	720,	726,	729,	736,	738,
	740,	742,	748,	750,	752,	754,	765,	768,	775,	777,
	780,	782,	793,	798,	810,	812,	814,	817,	826,	846,
	848,	858,	868,	880,	882,	891,	897,	899,	900,	901,
	910,	918,	923,	928,	936,	938,	943,	944,	950,	952,
	960,	966,	984,	986,	987,	988,	990,	992,	999,	1001,
	1003,	1005,	1012,	1020,	1022,	1023,	1024,	1025,	1027,	1032,
	1034,	1035,	1054,	1056,	1058,	1060,	1062,	1064,	1071,	1073,
	1078,	1079,	1081,	1083,	1098,	1100,	1102,	1104,	1105,	1116,
	1118,	1120,	1125,	1134,	1148,	1150,	1155,	1157,	1159,	1160,
	1166,	1168,	1173,	1176,	1178,	1183,	1188,	1204,	1207,	1216,
	1218,	1219,	1220,	1222,	1230,	1239,	1240,	1245,	1246,	1248,
	1258,	1269,	1271,	1272,	1273,	1274,	1276,	1278,	1288,	1290,
	1296,	1298,	1300,	1309,	1311,	1312,	1314,	1323,	1326,	1328,
	1330,	1332,	1333,	1340,	1342,	1350,	1358,	1363,	1375,	1376,
	1378,	1392,	1404,	1406,	1408,	1414,	1417,	1419,	1420,	1422,
	1424,	1426,	1428,	1440,	1442,	1443,	1450,	1455,	1456,	1458,
	1460,	1462,	1470,	1472,	1474,	1480,	1482,	1494,	1496,	1498,
	1513,	1515,	1517,	1518,	1519,	1534,	1537,	1539,	1545,	1548,
	1550,	1552,	1558,	1560,	1562,	1564,	1566,	1573,	1575,	1580,
	1582,	1584,	1591,	1593,	1596,	1598,	1600,	1606,	1608,	1610,
	1615,	1632,	1633,	1638,	1643,	1645,	1647,	1648,	1649,	1650,
	1651,	1664,	1672,	1675,	1677,	1679,	1680,	1681,	1690,	1692,
	1694,	1700,	1701,	1702,	1703,	1708,	1710,	1715,	1720,	1734,
	1736,	1738,	1740,	1742,	1744,	1746,	1748,	1749,	1750,	1767,
	1768,	1769,	1776,	1778,	1780,	1792,	1798,	1800,	1802,	1804,
	1805,	1806,	1807,	1813,	1815,	1817,	1818,	1824,	1825,	1827,
	1834,	1843,	1846,	1848,	1853,	1854,	1860,	1870,	1885,	1886,
	1888,	1890,	1896,	1898,	1909,	1917,	1918,	1919,	1920,	1922,
	1924,	1925,	1935,	1936,	1938,	1944,	1950,	1953,	1955,	1958,
	1960,	1961,	1963,	1972,	1974,	1976,	1978,	1998,	2000,	2013,
	2014,	2021,	2023,	2028,	2030,	2032,	2033,	2035,	2044,	2046,
	2048,	2050,	2052,	2055,	2057,	2058,	2065,	2068,	2070,	2072,
	2075,	2077,	2086,	2088,	2091,	2106,	2107,	2112,	2114,	2115,
	2117,	2119,	2128,	2130,	2132,	2133,	2134,	2140,	2142,	2144,
	2145,	2156,	2158,	2160,	2171,	2173,	2176,	2178,	2183,	2185,
	2190,	2196,	2198,	2208,	2209,	2210,	2212,	2220,	2222,	2223,
	2224,	2225,	2231,	2233,	2240,	2242,	2247,	2254,	2255,	2260,
	2261,	2263,	2266,	2275,	2277,	2278,	2280,	2286,	2289,	2294,
	2299,	2301,	2310,	2312,	2318,	2320,	2322,	2323,	2325,	2327,
	2329,	2336,	2338,	2343,	2350,	2352,	2353,	2354,	2356,	2358,
	2363,	2366,	2368,	2373,	2376,	2378,	2385,	2392,	2398,	2401,
	2403,	2405,	2407,	2412,	2413,	2414,	2416,	2418,	2420,	2422,
	2430,	2436,	2438,	2440,	2442,	2448,	2449,	2464,	2470,	2478,
	2480,	2482,	2483,	2484,	2489,	2490,	2492,	2494,	2499,	2500,
	2505,	2506,	2507,	2509,	2520,	2523,	2525,	2528,	2533,	2535,
	2538,	2540,	2542,	2544,	2546,	2548,	2550,	2553,	2560,	2565,
	2573,	2574,	2581,	2583,	2585,	2590,	2595,	2597,	2599,	2601,
	2608,	2616,	2618,	2620,	2622,	2623,	2624,	2625,	2626,	2639,
	2650,	2660,	2662,	2664,	2665,	2666,	2667,	2668,	2670,	2672,
	2673,	2678,	2680,	2682,	2684,	2686,	2695,	2698,	2700,	2701,
	2702,	2704,	2712,	2714,	2715,	2716,	2718,	2725,	2726,	2728,
	2736,	2737,	2738,	2743,	2745,	2750,	2752,	2754,	2756,	2758,
	2769,	2771,	2772,	2773,	2780,	2781,	2782,	2784,	2786,	2788,
	2793,	2806,	2812,	2814,	2820,	2826,	2829,	2832,	2838,	2840,
	2842,	2847,	2848,	2850,	2860,	2862,	2865,	2867,	2869,	2871,
	2873,	2875,	2880,	2881,	2882,	2884,	2886,	2888,	2889,	2896,
	2898,	2905,	2910,	2912,	2914,	2919,	2921,	2923,	2924,	2926,
	2938,	2941,	2943,	2948,	2950,	2952,	2958,	2968,	2975,	2980,
	2983,	2985,	2987,	2988,	2990,	3000,	3006,	3007,	3008,	3013,
	3014,	3024,	3034,	3036,	3038,	3040,	3043,	3048,	3050,	3051,
	3053,	3056,	3058,	3066,	3069,	3072,	3075,	3078,	3080,	3088,
	3094,	3100,	3102,	3103,	3105,	3114,	3120,	3122,	3125,	3127,
	3129,	3131,	3133,	3144,	3149,	3150,	3152,	3157,	3159,	3160,
	3162,	3171,	3172,	3173,	3178,	3180,	3182,	3184,	3190,	3193,
	3195,	3196,	3198,	3204,	3210,	3211,	3220,	3222,	3230,	3232,
	3237,	3243,	3245,	3247,	3248,	3256,	3258,	3260,	3262,	3264,
	3266,	3268,	3275,	3276,	3278,	3281,	3283,	3285,	3286,	3287,
	3288,	3294,	3296,	3298,	3300,	3302,	3304,	3315,	3318,	3320,
	3328,	3330,	3333,	3335,	3337,	3339,	3344,	3346,	3348,	3358,
	3360,	3362,	3364,	3374,	3379,	3380,	3381,	3382,	3388,	3390,
	3393,	3395,	3397,	3399,	3400,	3401,	3402,	3403,	3404,	3406,
	3408,	3417,	3420,	3422,	3423,	3425,	3427,	3429,	3430,	3432,
	3438,	3439,	3454,	3456,	3458,	3471,	3472,	3473,	3474,	3475,
	3476,	3477,	3478,	3479,	3481,	3483,	3484,	3486,	3495,	3496,
	3498,	3500,	3502,	3507,	3523,	3526,	3534,	3535,	3536,	3538,
	3540,	3549,	3550,	3551,	3553,	3560,	3562,	3565,	3567,	3568,
	3570,	3575,	3580,	3582,	3584,	3585,	3586,	3587,	3589,	3596,
	3597,	3598,	3599,	3608,	3610,	3612,	3614,	3616,	3618,	3619,
	3624,	3625,	3627,	3630,	3632,	3634,	3636,	3638,	3648,	3650,
	3652,	3653,	3655,	3658,	3664,	3666,	3672,	3680,	3683,	3690,
	3696,	3698,	3703,	3706,	3708,	3710,	3718,	3721,	3723,	3724,
	3726,	3729,	3735,	3738,	3741,	3743,	3749,	3750,	3751,	3757,
	3760,	3763,	3766,	3768,	3770,	3772,	3773,	3774,	3780,	3781,
	3782,	3792,	3794,	3796,	3798,	3799,	3806,	3807,	3808,	3809,
	3811,	3813,	3816,	3818,	3820,	3822,	3825,	3834,	3836,	3838,
	3840,	3841,	3842,	3848,	3850,	3852,	3854,	3855,	3857,	3859,
	3869,	3870,	3872,	3874,	3876,	3878,	3885,	3886,	3892,	3893,
	3904,	3910,	3912,	3913,	3914,	3920,	3922,	3924,	3925,	3934,
	3936,	3937,	3938,	3940,	3942,	3948,	3950,	3953,	3959,	3960,
	3969,	3971,	3973,	3975,	3976,	3977,	3978,	3980,	3982,	3991,
	3993,	4000,	4002,	4009,	4012,	4014,	4015,	4016,	4018,	4023,
	4026,	4028,	4029,	4030,	4031,	4033,	4040,	4046,	4048,	4059,
	4061,	4063,	4064,	4065,	4066,	4067,	4068,	4069,	4075,	4080,
	4085,	4086,	4087,	4092,	4094,	4095,	4100,	4102,	4108,	4110,
	4112,	4114,	4116,	4118,	4120,	4121,	4123,	4128,	4134,	4140,
	4142,	4148,	4150,	4152,	4158,	4161,	4163,	4170,	4171,	4173,
	4179,	4180,	4181,	4183,	4185,	4187,	4189,	4192,	4200,	4212,
	4214,	4216,	4218,	4221,	4223,	4228,	4230,	4232,	4233,	4235,
	4237,	4238,	4240,	4245,	4246,	4248,	4255,	4264,	4266,	4268,
	4270,	4275,	4278,	4288,	4290,	4292,	4294,	4296,	4298,	4300,
	4301,	4302,	4303,	4305,	4307,	4316,	4318,	4321,	4323,	4324,
	4329,	4332,	4334,	4336,	4338,	4340,	4342,	4343,	4344,	4345,
	4350,	4352,	4354,	4366,	4368,	4369,	4378,	4379,	4380,	4382,
	4384,	4387,	4392,	4393,	4394,	4399,	4400,	4401,	4408,	4416,
	4418,	4420,	4422,	4425,	4427,	4429,	4431,	4437,	4438,	4444,
	4446,	4448,	4450,	4452,	4453,	4460,	4462,	4464,	4469,	4470,
	4471,	4473,	4475,	4480,	4482,	4494,	4496,	4498,	4503,	4508,
	4509,	4510,	4512,	4518,	4520,	4522,	4524,	4526,	4528,	4530,
	4537,	4539,	4541,	4543,	4551,	4554,	4556,	4557,	4558,	4563,
	4573,	4575,	4576,	4577,	4578,	4580,	4582,	4585,	4587,	4588,
	4590,	4598,	4599,	4600,	4605,	4606,	4615,	4617,	4623,	4624,
	4626,	4632,	4633,	4634,	4640,	4642,	4644,	4646,	4648,	4653,
	4654,	4656,	4658,	4660,	4662,	4665,	4667,	4669,	4672,	4674,
	4681,	4686,	4688,	4690,	4693,	4695,	4697,	4698,	4704,	4706,
	4708,	4710,	4715,	4716,	4717,	4718,	4725,	4726,	4728,	4730,
	4732,	4734,	4736,	4738,	4743,	4745,	4750,	4753,	4755,	4756,
	4758,	4767,	4768,	4769,	4771,	4773,	4774,	4775,	4777,	4794,
	4795,	4800,	4805,	4807,	4809,	4814,	4816,	4818,	4823,	4824,
	4826,	4828,	4830,	4836,	4838,	4840,	4841,	4842,	4843,	4845,
	4847,	4848,	4849,	4851,	4858,	4860,	4862,	4865,	4876,	4878,
	4884,	4886,	4888,	4891,	4893,	4895,	4896,	4897,	4898,	4899,
	4900,	4912,	4920,	4925,	4927,	4928,	4930,	4940,	4942,	4945,
	4947,	4956,	4958,	4959,	4964,	4966,	4968,	4970,	4975,	4976,
	4977,	4978,	4979,	4980,	4981,	4982,	4984,	4986,	4992,	4995,
	4998,	5000,	5002,	5008,	5010,	5012,	5014,	5015,	5016,	5017,
	5018,	5020,	5022,	5024,	5029,	5035,	5038,	5041,	5044,	5047,
	5053,	5054,	5055,	5056,	5058,	5061,	5064,	5066,	5068,	5070,
	5072,	5073,	5074,	5080,	5082,	5083,	5092,	5094,	5104,	5109,
	5110,	5115,	5120,	5122,	5123,	5124,	5133,	5134,	5136,	5138,
	5141,	5143,	5146,	5148,	5149,	5150,	5157,	5159,	5160,	5166,
	5168,	5175,	5181,	5183,	5185,	5190,	5192,	5198,	5200,	5203,
	5205,	5206,	5211,	5213,	5214,	5215,	5217,	5219,	5220,	5236,
	5238,	5244,	5246,	5247,	5248,	5249,	5250,	5251,	5252,	5254,
	5256,	5258,	5263,	5265,	5278,	5280,	5285,	5287,	5289,	5290,
	5291,	5293,	5300,	5302,	5304,	5310,	5311,	5317,	5319,	5328,
	5330,	5332,	5334,	5336,	5338,	5339,	5341,	5344,	5346,	5348,
	5350,	5353,	5356,	5358,	5360,	5362,	5363,	5368,	5369,	5370,
	5371,	5373,	5375,	5377,	5380,	5390,	5392,	5394,	5395,	5400,
	5402,	5404,	5406,	5418,	5421,	5423,	5425,	5427,	5428,	5430,
	5432,	5434,	5445,	5446,	5448,	5450,	5451,	5452,	5454,	5457,
	5459,	5472,	5473,	5474,	5476,	5478,	5486,	5488,	5494,	5495,
	5497,	5500,	5502,	5508,	5510,	5511,	5516,	5518,	5520,	5523,
	5535,	5536,	5537,	5538,	5540,	5542,	5544,	5546,	5547,	5548,
	5549,	5550,	5558,	5559,	5560,	5562,	5564,	5565,	5566,	5575,
	5584,	5586,	5587,	5588,	5590,	5593,	5595,	5598,	5603,	5605,
	5607,	5609,	5610,	5612,	5614,	5616,	5618,	5624,	5625,	5626,
	5627,	5632,	5633,	5640,	5642,	5648,	5650,	5652,	5654,	5656,
	5658,	5660,	5662,	5668,	5671,	5673,	5676,	5678,	5685,	5694,
	5695,	5696,	5697,	5698,	5704,	5705,	5707,	5713,	5723,	5724,
	5725,	5726,	5727,	5728,	5733,	5734,	5736,	5738,	5740,	5742,
	5744,	5757,	5760,	5762,	5764,	5766,	5771,	5773,	5775,	5776,
	5778,	5780,	5782,	5784,	5790,	5795,	5796,	5798,	5808,	5809,
	5817,	5820,	5822,	5828,	5829,	5830,	5831,	5833,	5835,	5838,
	5840,	5846,	5848,	5850,	5856,	5858,	5863,	5866,	5872,	5874,
	5880,	5883,	5885,	5886,	5887,	5888,	5889,	5891,	5893,	5894,
	5899,	5900,	5902,	5907,	5908,	5909,	5911,	5913,	5915,	5916,
	5918,	5920,	5934,	5941,	5943,	5945,	5950,	5952,	5955,	5957,
	5958,	5960,	5962,	5964,	5966,	5967,	5968,	5973,	5974,	5975,
	5976,	5978,	5983,	5984,	5992,	5993,	6001,	6004,	6012,	6014,
	6016,	6018,	6019,	6021,	6023,	6024,	6025,	6026,	6028,	6030,
	6032,	6040,	6048,	6050,	6058,	6059,	6060,	6061,	6062,	6064,
	6066,	6069,	6075,	6084,	6094,	6095,	6097,	6100,	6102,	6103,
	6104,	6105,	6107,	6109,	6110,	6116,	6118,	6123,	6135,	6136,
	6138,	6140,	6142,	6144,	6146,	6148,	6154,	6160,	6162,	6169,
	6174,	6175,	6177,	6180,	6192,	6200,	6202,	6204,	6206,	6208,
	6213,	6214,	6220,	6222,	6223,	6224,	6225,	6226,	6228,	6230,
	6231,	6233,	6235,	6237,	6239,	6244,	6246,	6248,	6251,	6253,
	6254,	6256,	6258,	6260,	6262,	6264,	6270,	6273,	6278,	6280,
	6283,	6286,	6288,	6289,	6290,	6291,	6298,	6300,	6302,	6303,
	6305,	6312,	6313,	6318,	6319,	6320,	6322,	6325,	6328,	6340,
	6342,	6344,	6346,	6348,	6350,	6352,	6354,	6355,	6356,	6358,
	6363,	6368,	6369,	6370,	6384,	6390,	6392,	6394,	6396,	6398,
	6399,	6400,	6401,	6403,	6405,	6407,	6417,	6420,	6422,	6424,
	6435,	6437,	6438,	6439,	6441,	6444,	6446,	6448,	6454,	6460,
	6461,	6462,	6463,	6464,	6465,	6466,	6475,	6477,	6478,	6480,
	6487,	6493,	6494,	6495,	6496,	6497,	6498,	6499,	6507,	6509,
	6510,	6511,	6517,	6524,	6525,	6526,	6531,	6532,	6534,	6536,
	6538,	6539,	6550,	6552,	6554,	6555,	6560,	6565,	6570,	6572,
	6573,	6574,	6576,	6578,	6583,	6586,	6591,	6592,	6594,	6601,
	6604,	6606,	6613,	6615,	6620,	6622,	6623,	6625,	6631,	6634,
	6640,	6641,	6642,	6643,	6645,	6647,	6648,	6655,	6664,	6665,
	6667,	6670,	6672,	6678,	6680,	6681,	6683,	6688,	6690,	6693,
	6695,	6696,	6697,	6698,	6704,	6705,	6706,	6708,	6713,	6714,
	6721,	6723,	6724,	6725,	6726,	6732,	6734,	6739,	6745,	6747,
	6748,	6749,	6750,	6752,	6755,	6758,	6760,	6762,	6764,	6765,
	6766,	6771,	6773,	6775,	6783,	6784,	6790,	6792,	6798,	6800,
	6802,	6804,	6806,	6808,	6810,	6815,	6816,	6818,	6820,	6832,
	6838,	6840,	6842,	6844,	6845,	6846,	6848,	6851,	6853,	6858,
	6860,	6862,	6868,	6873,	6874,	6875,	6876,	6878,	6880,	6885,
	6886,	6887,	6888,	6893,	6894,	6895,	6900,	6901,	6913,	6923,
	6928,	6929,	6940,	6942,	6943,	6944,	6946,	6948,	6950,	6951,
	6952,	6953,	6954,	6956,	6958,	6960,	6963,	6966,	6968,	6970,
	6972,	6973,	6974,	6984,	6985,	6987,	6990,	6992,	6994,	7000,
	7002,	7003,	7004,	7005,	7006,	7007,	7009,	7018,	7028,	7029,
	7030,	7031,	7033,	7035,	7038,	7040,	7047,	7050,	7059,	7060,
	7065,	7068,	7070,	7072,	7074,	7075,	7085,	7087,	7093,	7095,
	7097,	7098,	7099,	7100,	7102,	7104,	7105,	7110,	7123,	7124,
	7126,	7128,	7134,	7136,	7137,	7138,	7139,	7144,	7150,	7152,
	7153,	7155,	7161,	7168,	7169,	7170,	7172,	7174,	7176,	7178,
	7180,	7182,	7189,	7192,	7194,	7196,	7198,	7199,	7200,	7201,
	7203,	7208,	7216,	7218,	7220,	7221,	7222,	7223,	7225,	7228,
	7230,	7236,	7238,	7239,	7240,	7248,	7250,	7252,	7257,	7258,
	7259,	7261,	7264,	7266,	7275,	7277,	7280,	7285,	7287,	7289,
	7291,	7294,	7296,	7298,	7301,	7303,	7304,	7306,	7308,	7310,
	7312,	7313,	7314,	7315,	7317,	7320,	7325,	7326,	7327,	7332,
	7334,	7335,	7337,	7345,	7348,	7358,	7359,	7360,	7361,	7362,
	7363,	7364,	7365,	7366,	7368,	7370,	7371,	7373,	7376,	7378,
	7379,	7381,	7383,	7385,	7395,	7396,	7397,	7398,	7406,	7408,
	7410,	7412,	7414,	7421,	7423,	7424,	7434,	7437,	7440,	7446,
	7448,	7450,	7453,	7456,	7458,	7460,	7462,	7467,	7469,	7470,
	7471,	7472,	7473,	7474,	7480,	7486,	7488,	7491,	7498,	7500,
	7501,	7502,	7503,	7504,	7512,	7518,	7519,	7524,	7525,	7530,
	7531,	7533,	7540,	7542,	7543,	7544,	7546,	7548,	7550,	7560,
	7562,	7564,	7565,	7566,	7567,	7569,	7578,	7579,	7580,	7582,
	7585,	7588,	7590,	7595,	7596,	7597,	7598,	7600,	7611,	7612,
	7614,	7616,	7618,	7620,	7623,	7625,	7630,	7631,	7632,	7633,
	7634,	7636,	7638,	7648,	7654,	7656,	7657,	7658,	7659,	7661,
	7663,	7665,	7668,	7670,	7676,	7678,	7680,	7682,	7683,	7684,
	7686,	7693,	7695,	7705,	7707,	7708,	7710,	7712,	7718,	7720,
	7733,	7738,	7740,	7742,	7747,	7748,	7749,	7750,	7756,	7758,
	7769,	7770,	7772,	7774,	7775,	7781,	7783,	7784,	7785,	7786,
	7788,	7790,	7792,	7798,	7800,	7801,	7803,	7805,	7807,	7808,
	7809,	7810,	7811,	7813,	7824,	7826,	7828,	7830,	7831,	7837,
	7843,	7845,	7847,	7848,	7850,	7852,	7854,	7857,	7866,	7869,
	7872,	7874,	7876,	7878,	7880,	7882,	7885,	7887,	7888,	7889,
	7896,	7898,	7900,	7902,	7904,	7910,	7911,	7918,	7920,	7921,
	7922,	7923,	7936,	7938,	7940,	7942,	7943,	7944,	7950,	7952,
	7953,	7954,	7955,	7956,	7957,	7958,	7959,	7964,	7965,	7966,
	7967,	7973,	7979,	7980,	7982,	7984,	7990,	7992,	7998,	7999,
	8001,	8008,	8016,	8018,	8019,	8020,	8021,	8022,	8023,	8025,
	8027,	8028,	8030,	8032,	8034,	8041,	8046,	8048,	8050,	8056,
	8058,	8060,	8062,	8064,	8073,	8075,	8078,	8080,	8082,	8083,
	8084,	8086,	8092,	8094,	8107,	8113,	8118,	8120,	8125,	8126,
	8128,	8130,	8132,	8134,	8136,	8138,	8140,	8142,	8143,	8148,
	8149,	8151,	8153,	8160,	8162,	8170,	8172,	8174,	8175,	8176,
	8178,	8181,	8188,	8190,	8192,	8201,	8203,	8204,	8205,	8206,
	8211,	8213,	8214,	8215,	8216,	8218,	8224,	8225,	8226,	8227,
	8232,	8234,	8239,	8240,	8242,	8244,	8249,	8251,	8253,	8255,
	8260,	8272,	8275,	8277,	8279,	8280,	8281,	8282,	8294,	8296,
	8298,	8300,	8302,	8303,	8305,	8308,	8319,	8320,	8321,	8323,
	8326,	8328,	8334,	8336,	8337,	8338,	8339,	8340,	8341,	8342,
	8344,	8347,	8350,	8355,	8357,	8358,	8360,	8362,	8364,	8366,
	8368,	8370,	8372,	8374,	8376,	8378,	8379,	8380,	8381,	8382,
	8383,	8384,	8386,	8393,	8395,	8397,	8398,	8399,	8404,	8411,
	8413,	8414,	8415,	8416,	8418,	8424,	8425,	8426,	8428,	8430,
	8435,	8437,	8439,	8440,	8446,	8448,	8449,	8451,	8453,	8464,
	8466,	8468,	8470,	8471,	8472,	8473,	8474,	8475,	8476,	8477,
	8478,	8479,	8481,	8487,	8490,	8492,	8494,	8496,	8498,	8500,
	8505,	8507,	8510,	8512,	8515,	8526,	8531,	8532,	8533,	8534,
	8547,	8549,	8554,	8555,	8556,	8557,	8558,	8565,	8567,	8580,
	8582,	8584,	8585,	8586,	8587,	8589,	8591,	8593,	8600,	8602,
	8604,	8606,	8608,	8611,	8614,	8616,	8618,	8619,	8620,	8622,
	8624,	8631,	8633,	8636,	8638,	8640,	8643,	8646,	8648,	8649,
	8650,	8651,	8653,	8656,	8658,	8660,	8662,	8664,	8666,	8668,
	8670,	8671,	8673,	8683,	8684,	8685,	8686,	8688,	8690,	8692,
	8695,	8700,	8702,	8708,	8710,	8715,	8720,	8721,	8723,	8730,
	8732,	8734,	8736,	8738,	8740,	8748,	8750,	8755,	8756,	8757,
	8758,	8760,	8762,	8764,	8766,	8769,	8771,	8772,	8773,	8775,
	8778,	8784,	8785,	8786,	8787,	8788,	8789,	8790,	8797,	8798,
	8799,	8800,	8802,	8804,	8806,	8809,	8814,	8820,	8823,	8825,
	8827,	8832,	8833,	8834,	8836,	8838,	8841,	8843,	8848,	8851,
	8853,	8854,	8855,	8856,	8858,	8860,	8862,	8864,	8873,	8874,
	8875,	8876,	8877,	8878,	8879,	8881,	8888,	8890,	8895,	8901,
	8903,	8904,	8906,	8908,	8909,	8910,	8918,	8925,	8928,	8930,
	8938,	8940,	8942,	8944,	8947,	8950,	8952,	8955,	8957,	8959,
	8965,	8968,	8974,	8975,	8976,	8977,	8978,	8980,	8983,	8985,
	8987,	8988,	8989,	8990,	8991,	8993,	8996,	8998,	9000,	9006,
	9008,	9015,	9016,	9017,	9018,	9019,	9020,	9022,	9025,	9028,
	9040,	9042,	9045,	9048,	9050,	9052,	9054,	9056,	9058,	9060,
	9061,	9062,	9063,	9071,	9072,	9073,	9075,	9078,	9080,	9083,
	9085,	9086,	9087,	9088,	9093,	9099,	9100,	9102,	9104,	9106,
	9112,	9114,	9116,	9118,	9126,	9128,	9129,	9139,	9140,	9141,
	9142,	9143,	9144,	9150,	9152,	9154,	9156,	9158,	9160,	9162,
	9163,	9165,	9167,	9168,	9174,	9175,	9176,	9177,	9178,	9191,
	9193,	9195,	9196,	9198,	9200,	9202,	9205,	9210,	9211,	9212,
	9213,	9214,	9215,	9216,	9217,	9218,	9220,	9223,	9230,	9232,
	9238,	9243,	9246,	9248,	9250,	9251,	9252,	9253,	9254,	9259,
	9260,	9261,	9262,	9263,	9264,	9266,	9268,	9269,	9270,	9271,
	9273,	9280,	9282,	9285,	9299,	9301,	9306,	9307,	9308,	9310,
	9313,	9315,	9320,	9322,	9328,	9329,	9331,	9334,	9336,	9338,
	9340,	9342,	9345,	9348,	9350,	9359,	9360,	9367,	9372,	9373,
	9374,	9376,	9378,	9379,	9384,	9387,	9390,	9392,	9394,	9396,
	9398,	9399,	9405,	9407,	9412,	9414,	9415,	9416,	9418,	9420,
	9422,	9423,	9430,	9432,	9434,	9436,	9438,	9450,	9451,	9452,
	9453,	9454,	9455,	9460,	9462,	9469,	9475,	9476,	9478,	9480,
	9481,	9483,	9485,	9486,	9488,	9490,	9492,	9504,	9510,	9512,
	9513,	9514,	9515,	9517,	9520,	9523,	9526,	9529,	9534,	9536,
	9538,	9540,	9546,	9548,	9550,	9552,	9553,	9555,	9557,	9558,
	9560,	9568,	9575,	9577,	9579,	9581,	9583,	9588,	9589,	9590,
	9591,	9592,	9593,	9595,	9597,	9603,	9605,	9610,	9612,	9614,
	9615,	9616,	9618,	9620,	9622,	9628,	9630,	9633,	9635,	9636,
	9637,	9638,	9639,	9646,	9648,	9650,	9656,	9657,	9658,	9664,
	9669,	9671,	9672,	9673,	9674,	9680,	9681,	9682,	9683,	9684,
	9686,	9688,	9693,	9694,	9696,	9698,	9699,	9700,	9701,	9703,
	9705,	9707,	9709,	9715,	9716,	9718,	9723,	9725,	9727,	9728,
	9729,	9735,	9738,	9744,	9750,	9756,	9758,	9761,	9763,	9766,
	9772,	9773,	9774,	9780,	9790,	9792,	9794,	9795,	9796,	9797,
	9798,	9799,	9800,	9802,	9805,	9807,	9808,	9810,	9812,	9814,
	9821,	9823,	9824,	9825,	9826,	9840,	9841,	9842,	9843,	9844,
	9845,	9847,	9853,	9854,	9856,	9858,	9867,	9870,	9875,	9877,
	9878,	9879,	9880,	9882,	9888,	9889,	9891,	9898,	9904,	9909,
	9911,	9913,	9915,	9916,	9917,	9918,	9920,	9926,	9928,	9930,
	9943,	9945,	9950,	9951,	9952,	9953,	9954,	9956,	9958,	9960,
	9962,	9964,	9966,	9980,	9982,	9988,	9994,	9997],

'genRandomPQRS': function(maxRS) {

	maxRS = Math.min (maxRS || this.MaxNumber, this.MaxNumber);

	var index = Math.floor(Math.random() * this.Pairs.length);

	while (this.Pairs[index]+2 > maxRS) {

		index = Math.floor(Math.random() * index);
	}

	var pq = this.findTwoFactors (this.Pairs[index]  );
	var rs = this.findTwoFactors (this.Pairs[index]+2);

	return [pq[0], pq[1], rs[0], rs[1]];
},

//------------------------------------------------------------------------------
//	random sum of n summands
//------------------------------------------------------------------------------

'genRandomSum': function(n, minValue, maxValue) {

	n        = n || 1;
	minValue = Math.max (minValue || this.MinNumber, this.MinNumber);
	maxValue = Math.min (maxValue || this.MaxNumber, this.MaxNumber);

	var range = maxValue - minValue*n + 1;

	for (var tries=1; tries<=this.MaxTries * n; tries++) {

		var summands = [];
		var sum = 0;

		for (var i=1; i<=n; i++) {

			var summand = minValue + Math.floor(Math.random()*range);
			summands.push(summand);
			sum += summand;
		}

		if (sum <= maxValue) return summands;
	}

	summands = [];

	for (i=1; i<=n; i++) {

		summands.push(Math.floor(minValue + Math.random()*range/n));
	}

	return summands;
},

//------------------------------------------------------------------------------
//	random product of n factors
//------------------------------------------------------------------------------

'genRandomProduct': function(n, minValue, maxValue) {

	n        = n || 1;
	minValue = Math.max (minValue || this.MinNumber, this.MinNumber);
	maxValue = Math.min (maxValue || this.MaxNumber, this.MaxNumber);

	var log = Math.log (maxValue+1) - Math.log(minValue)*n;

	for (var tries=1; tries<=this.MaxTries * n; tries++) {

		var factors = [];
		var product = 1;

		for (var i=1; i<=n; i++) {

			var factor = Math.floor (minValue * Math.exp(Math.random()*log));
			factors.push(factor);
			product *= factor;
		}

		if (product <= maxValue) return factors;
	}

	factors = [];

	for (i=1; i<=n; i++) {

		factors.push(Math.floor (minValue * Math.exp(Math.random()*log/n)));
	}

	return factors;
},

//------------------------------------------------------------------------------
//	random product of 2 factors
//------------------------------------------------------------------------------

'genRandomProduct2': function(minValue, maxValue) {

	minValue = Math.max (minValue || this.MinNumber, this.MinNumber);
	maxValue = Math.min (maxValue || this.MaxNumber, this.MaxNumber);

	var scale = (maxValue+1) / minValue / minValue;

	var p = Math.floor (minValue * Math.pow(scale, Math.random()));
	var q = Math.floor (minValue + Math.random() * (maxValue/p-minValue));

	return [p, q];
},

//------------------------------------------------------------------------------
//	Typed generators
//------------------------------------------------------------------------------

'generator_000': function() {	// +++ +++

	var abde = this.genRandomSum (4);
	var a = abde[0];
	var b = abde[1];
	var d = abde[2];
	var e = abde[3];

	return [a, b, a+b, d, e, d+e, a+d, b+e, a+b+d+e];
},

'generator_006': function() {	// *-+ +++	(Z03)

	var pq = this.genRandomProduct2 (this.MinNumber+2);

	var p = pq[0];
	var q = pq[1];

	var a = (p-1) & ~1;	// !!even
	var d = (q-1) & ~1;	// !!even
	var e = Math.floor((a*d-a-d)/2);
	var h = Math.floor(this.MinNumber+Math.random()*(this.MaxNumber-this.MinNumber+1-a*d));

	return [a, e+h, a+e+h, d, e, d+e, a*d, h, a*d+h];
},

'generator_00b': function() {	// /*+ +*+

	var pqrs = this.genRandomPQRS (this.MaxNumber/2);

	var p = pqrs[0];
	var q = pqrs[1];
	var r = pqrs[2];
	var s = pqrs[3];

	var d = p+1;
	var g = q-1;
	var b = r+1;
	var e = s+1;

	return [d*g, b, d*g+b, d, e, d+e, g, b*e, g+b*e];
},

'generator_092': function() {	// *+- *++	(=Pousen)

	var ba = this.genRandomProduct2 (12);
	var b = ba[0];
	var a = ba[1];

	var d = this.genRandomSum(1,this.MinNumber+1,((a-1)*b-1)/(a+1))[0];
	if (!(a&1) && ((b&1) != (d&1))) d--;

	var e = Math.floor ((a * (b-d) - b - d) / 2);

	return [a, b, a*b, d, e, d+e, a*d, b+e, a*d+b+e];
},

'generator_104': function() {	// +-+ +-+

	var aefh = this.genRandomSum (4);

	var a = aefh[0];
	var e = aefh[1];
	var f = aefh[2];
	var h = aefh[3];

	return [a, e+h, a+e+h, e+f, e, f, a+e+f, h, a+e+f+h];
},

'generator_29a': function() {	// **- **+

	var pqrs = this.genRandomProduct (4, 5);

	var p = pqrs[0];
	var q = pqrs[1];
	var r = pqrs[2];
	var s = pqrs[3];

	var f = Math.random() + 1;

	p = Math.floor(Math.sqrt(p*r/f) - 1);
	r = Math.floor(p*f);

	if (r >= 2*p-1) r = 2*p-1;
	if (r <=   p  ) r =   p+1;

	var a = p*q;
	var b = r*s;
	var d = (2*p-r)*s;
	var e = (r-p)*q;

	return [a, b, a*b, d, e, d*e, a*d, b*e, a*d+b*e];
},

'generator_418': function() {	// +*- ++-	(z02)

	var pq = this.genRandomProduct2 (this.MinNumber+1, this.MaxNumber/4);
	var p = pq[0] - 1;
	var q = pq[1];

	var b = 2*p;
	var d = 2*p*q+p-q;
	var e = 2*q;

	var a = this.genRandomSum (1, 2*p*q-p+q, this.MaxNumber-d)[0];

	return [a, b, a+b, d, e, d+e, a+d, b*e, a+b-d-e];
},

'generator_aaa': function() {	// *** ***

	var abde = this.genRandomProduct (4);

	var a = abde[0];
	var b = abde[1];
	var d = abde[2];
	var e = abde[3];

	return [a, b, a*b, d, e, d*e, a*d, b*e, a*b*d*e];
},

'generator_bae': function() {	// */* */*

	var aefh = this.genRandomProduct (4);

	var a = aefh[0];
	var e = aefh[1];
	var f = aefh[2];
	var h = aefh[3];

	return [a, e*h, a*e*h, e*f, e, f, a*e*f, h, a*e*f*h];
},

//------------------------------------------------------------------------------
//	generator catalogue
//------------------------------------------------------------------------------

'TypeToMasterType': null,
'TypeToVariantId': null,
'Types': null,
'generators': null,

//------------------------------------------------------------------------------
//	initializer
//------------------------------------------------------------------------------

'initialize': function() {

	this.generators = {
		'000': this.generator_000,
		'006': this.generator_006,
		'00b': this.generator_00b,
		'092': this.generator_092,
		'104': this.generator_104,
		'29a': this.generator_29a,
		'418': this.generator_418,
		'aaa': this.generator_aaa,
		'bae': this.generator_bae
	};

	this.TypeToMasterType = {};
	this.TypeToVariantId  = {};
	this.Types            = [];

	function o1(op) { return (op>>0) & 3; }
	function o2(op) { return (op>>2) & 3; }
	function o3(op) { return (op>>4) & 3; }

	function op(op1,op2,op3) { return 1*(op1&3) + 4*(op2&3) + 16*(op3&3); }

	for (var masterType in this.generators) {

		var ops = parseInt(masterType, 16);

		for (var variant=0x00; variant<=0x7f; variant++) {

		//--------------------------------------------------------------
		//		split column and row operators
		//--------------------------------------------------------------

			var rowOps = ops >> 6;
			var colOps = ops % 64;

		//--------------------------------------------------------------
		//		(01) swap 1st and 3rd row, invert col ops
		//--------------------------------------------------------------

			if (variant & 0x01) {

				colOps ^= 0x15;
				rowOps = op (o3(rowOps),o2(rowOps),o1(rowOps));
			}

		//--------------------------------------------------------------
		//		(02) swap 1st and 3rd col, invert row ops
		//--------------------------------------------------------------

			if (variant & 0x02) {

				rowOps ^= 0x15;
				colOps = op (o3(colOps),o2(colOps),o1(colOps));
			}

		//--------------------------------------------------------------
		//		(04) transpose
		//--------------------------------------------------------------

			if (variant & 0x04) {

				var tmp = rowOps;
				rowOps  = colOps;
				colOps  = tmp;
			}

		//--------------------------------------------------------------
		//		(08) swap 1st and 2nd col (if rowOps symm.)
		//--------------------------------------------------------------

			if (variant & 0x08) {

				if ((rowOps & 0x15) !== 0x00) continue;
				colOps = op (o2(colOps),o1(colOps),o3(colOps));
			}

		//--------------------------------------------------------------
		//		(10) swap 2nd and 3rd col (if rowOps antisymm.)
		//--------------------------------------------------------------

			if (variant & 0x10) {

				if ((rowOps & 0x15) !== 0x15) continue;
				colOps = op (o1(colOps),o3(colOps),o2(colOps));
			}

		//--------------------------------------------------------------
		//		(20) swap 1st and 2nd row (if colOps symm.)
		//--------------------------------------------------------------

			if (variant & 0x20) {

				if ((colOps & 0x15) !== 0x00) continue;
				rowOps = op (o2(rowOps),o1(rowOps),o3(rowOps));
			}

		//--------------------------------------------------------------
		//		(40) swap 2nd and 3rd row (if colOps antisymm.)
		//--------------------------------------------------------------

			if (variant & 0x40) {

				if ((colOps & 0x15) !== 0x15) continue;
				rowOps = op (o1(rowOps),o3(rowOps),o2(rowOps));
			}

		//--------------------------------------------------------------
		//		combine operators
		//--------------------------------------------------------------

			var variantOps = rowOps * 64 + colOps;
			var type = ('00'+variantOps.toString(16)).substr(-3);

			if (this.TypeToMasterType[type]) continue;

			this.TypeToMasterType[type] = masterType;
			this.TypeToVariantId [type] = variant;
			this.Types.push(type);
		}
	}

	this.Types.sort();
},

//------------------------------------------------------------------------------
//	generate random numbers and operators 
//------------------------------------------------------------------------------

'genPuzzle': function() {

	if (!this.Types) this.initialize();

	var type    = this.Types[Math.floor(Math.random() * this.Types.length)];
	var master  = this.TypeToMasterType[type];
	var vari    = this.TypeToVariantId [type];

	var n = this.generators[master].apply(this);

	if (vari & 0x01) n = [n[6],n[7],n[8],n[3],n[4],n[5],n[0],n[1],n[2]];
	if (vari & 0x02) n = [n[2],n[1],n[0],n[5],n[4],n[3],n[8],n[7],n[6]];
	if (vari & 0x04) n = [n[0],n[3],n[6],n[1],n[4],n[7],n[2],n[5],n[8]];
	if (vari & 0x08) n = [n[1],n[0],n[2],n[4],n[3],n[5],n[7],n[6],n[8]];
	if (vari & 0x10) n = [n[0],n[2],n[1],n[3],n[5],n[4],n[6],n[8],n[7]];
	if (vari & 0x20) n = [n[3],n[4],n[5],n[0],n[1],n[2],n[6],n[7],n[8]];
	if (vari & 0x40) n = [n[0],n[1],n[2],n[6],n[7],n[8],n[3],n[4],n[5]];

	var id = [type, 10, n[0], n[1], n[3], n[4]].join('-');

	var ops = parseInt(type, 16);

	return {
		'id': id,
		'rowOps':  [ (ops>> 6) % 4, (ops>> 8) % 4, (ops>>10) % 4],
		'colOps':  [ (ops>> 0) % 4, (ops>> 2) % 4, (ops>> 4) % 4],
		'numbers': [n[0], n[1], n[2], n[3], n[4], n[5], n[6], n[7], n[8]],
		'verify':     this.verifyPuzzle,
		'toString':   this.toStringPuzzle,
		'genSymbols': this.genSymbolsPuzzle
	};
},

//------------------------------------------------------------------------------
//	puzzle: verify
//------------------------------------------------------------------------------

'verifyPuzzle': function() {

	function ck(o, a, b, c) {

		if (o===0) return a + b === c;
		if (o===1) return c + b === a;
		if (o===2) return a * b === c;
		if (o===3) return c * b === a;
		return false;
	}

	if (!ck(this.rowOps[0], this.numbers[0], this.numbers[1], this.numbers[2])) return false;
	if (!ck(this.rowOps[1], this.numbers[3], this.numbers[4], this.numbers[5])) return false;
	if (!ck(this.rowOps[2], this.numbers[6], this.numbers[7], this.numbers[8])) return false;

	if (!ck(this.colOps[0], this.numbers[0], this.numbers[3], this.numbers[6])) return false;
	if (!ck(this.colOps[1], this.numbers[1], this.numbers[4], this.numbers[7])) return false;
	if (!ck(this.colOps[2], this.numbers[2], this.numbers[5], this.numbers[8])) return false;

	return true;
},

//------------------------------------------------------------------------------
//	puzzle: toString
//------------------------------------------------------------------------------

toStringPuzzle: function() {

	function d4(n) { return ('\xa0\xa0\xa0'+n).substr(-4); }
	var op = ['+', '-', '×', '÷'];

	return [
		this.id,
		this.rowOps.join(','),
		this.colOps.join(','),
		this.numbers.join(','),
		"",
		d4(this.numbers[0]) + ' ' + op[this.rowOps[0]] + ' ' + d4(this.numbers[1]) + ' = ' + d4(this.numbers[2]),
		'   ' + op[this.colOps[0]] + '      ' + op[this.colOps[1]] + '      ' + op[this.colOps[2]],
		d4(this.numbers[3]) + ' ' + op[this.rowOps[1]] + ' ' + d4(this.numbers[4]) + ' = ' + d4(this.numbers[5]),
		'   =      =      =',
		d4(this.numbers[6]) + ' ' + op[this.rowOps[2]] + ' ' + d4(this.numbers[7]) + ' = ' + d4(this.numbers[8])
	].join('\n');
},

//------------------------------------------------------------------------------
//	puzzle: genSymbols
//------------------------------------------------------------------------------

'genSymbolsPuzzle': function(digitSymbols, random) {

	digitSymbols = digitSymbols.slice();

	var digitToSymbolMap = {};
	var symbols          = [];

	for (var i=0; i<this.numbers.length; i++) {

		var digits = this.numbers[i].toString().split('');

		var symbol = [];

		for (var j=0; j<digits.length; j++) {

			var digit = digits[j];
			var digitSymbol = digitToSymbolMap[digit];

			if (typeof(digitSymbol)=='undefined') {

				var x = random? Math.random()*digitSymbols.length : 0;
				digitSymbol = digitSymbols.splice(x, 1)[0];
				digitToSymbolMap[digit] = digitSymbol;
			}

			symbol.push(digitSymbol);
		}

		symbols.push(symbol);
	}

	this.symbols = symbols;
	return symbols;
},

//------------------------------------------------------------------------------
//	selftest
//------------------------------------------------------------------------------

'selftest': function(nRounds) {

	for(var round=1; round <= nRounds; round++) {

		var puzzle = this.genPuzzle();
		var ok = puzzle.verify();

		do {
			window.alert(ok + '\n\n' + puzzle.toString());
		} while(!ok);
	}
}

};

//------------------------------------------------------------------------------
//	$Id: gen-zahlenraetsel.js,v 1.2 2019/01/01 04:12:54 wolf Exp $
//------------------------------------------------------------------------------
